/***********************************************************************/
/*                                                                     */
/*                              Pio (C++)                              */
/*                                                                     */
/*                    Polyglot Input/Output Library                    */
/*                                                                     */
/*                  projet Estime, INRIA-Rocquencourt                  */
/*                                                                     */
/*  Copyright 2004-2011 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                       */
/*                                                                     */
/***********************************************************************/

/********************************************************************************/
/*                                                                              */
/*                                  Pio (C++)                                   */
/*                                                                              */
/*                        Polyglot Input/Output Library                         */
/*                                                                              */
/*                      projet Estime, INRIA-Rocquencourt                       */
/*                                                                              */
/*  Copyright 2004-2011 INRIA.                                                  */
/*  Institut National de Recherche en Informatique et en Automatique.           */
/*  All rights reserved. This file is distributed only by permission.           */
/*                                                                              */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                                */
/*                                                                              */
/********************************************************************************/

/***********************************************************************/
/*                                                                     */
/*                              Io (C++)                               */
/*                                                                     */
/*                  projet Estime, INRIA-Rocquencourt                  */
/*                                                                     */
/*  Copyright 2005-2006 INRIA.                                         */
/*  Institut National de Recherche en Informatique et en Automatique.  */
/*  All rights reserved. This file is distributed only by permission.  */
/*                                                                     */
/*  Estelle Marchand <Estelle.Marchand@inria.fr>                       */
/*                                                                     */
/***********************************************************************/

/* $Id: single_types.hpp,v 1.3 2011-02-23 17:43:29 fclement Exp $ */

#include<iostream>
#include<vector>
#include<algorithm>
#include<fstream>
#include<sstream>
#include<utility>

#include "global.hpp"
#include "boolean.hpp"

#ifndef SINGLE_TYPES_H
#define SINGLE_TYPES_H
using namespace std;
class TupleRangeError{};

template<typename dataType,int n> class Tuple{
  vector<dataType> _coord;
public:
  Tuple():_coord(n){}
  Tuple(vector<dataType> v){
    if(v.size() == n){
      _coord = v;
    }
    else{
      cerr<<"error: argument of size "<<v.size()<<" for Tuple<"<<n<<"> constructor" <<endl;
      TupleRangeError t;throw t;
    }
  }
  virtual ~Tuple(){
  }
  Tuple<dataType,n>(const Tuple<dataType, n>& f):_coord(f._coord){}
  dataType& operator [] (int m){
    if(m<0 || m>=n){
      cerr<<"error: out of range indice "<<m<<" in Tuple<"<<n<<">."<<endl;
      TupleRangeError t;throw(t);
    }
    return (_coord[m]);
  }
  const dataType& operator [] (int m) const{
    if(m<0 || m>=n){
      cerr<<"error: out of range indice "<<m<<" in Tuple<"<<n<<">."<<endl;
      TupleRangeError t;throw(t);
    }
    return (_coord[m]);
  }
  dataType& operator () (int m){
    if(m<0 || m>=n){
      cerr<<"error: out of range indice "<<m<<" in Tuple<"<<n<<">."<<endl;
      TupleRangeError t;throw(t);
    }
    return (_coord[m]);
  }
  const dataType& operator () (int m) const{
    if(m<0 || m>=n){
      cerr<<"error: out of range indice "<<m<<" in Tuple<"<<n<<">."<<endl;
      TupleRangeError t;throw(t);
    }
    return (_coord[m]);
  }
  void set_value(int i,dataType x){
    _coord[i]=x;
  }
  vector<dataType> to_vector(){return _coord;}
};

typedef Tuple<double,3> float_triple;
typedef Tuple<double,2> float_couple;
typedef Tuple<int,3> int_triple;
typedef Tuple<int,2> int_couple;
typedef Tuple<boolean,3> boolean_triple;
typedef Tuple<boolean,2> boolean_couple;

//template<typename dataType>
//typedef
//Tuple<dataType,3>
//Triple<dataType>;


typedef pair<int,double> int_float_couple;
typedef pair<double,int> float_int_couple;
typedef pair<int,string> int_string_couple;
typedef pair<string,int> string_int_couple;
typedef pair<string,double> string_float_couple;
typedef pair<double,string> float_string_couple;

namespace io{
 
  enum Single_type{EMPTY,INT,FLOAT,STRING,BOOLEAN,FLOAT_TRIPLE,FLOAT_COUPLE,INT_TRIPLE,INT_COUPLE,BOOLEAN_TRIPLE,BOOLEAN_COUPLE};

  enum Aim{SEND,RECEIVE};

  string type(const int& i, Aim aim);
  string type(const double& d, Aim aim);
  string type(const string& S, Aim aim);
  string type(const boolean& b, Aim aim);
  template<class dataType,int n> string type(const Tuple<dataType,n>& t,Aim aim){
    dataType tt;
    string s=type(tt,aim);
    string res= "(";
    for(int i=0;i<n-1;i++)
      res += s+", ";
    res += s+')';
    return res;
  }
  //template<class dataType1,class dataType2> string type(pair<dataType1,dataType2>& t);
  template<class dataType1,class dataType2> string type(const pair<dataType1,dataType2>& t,Aim aim){
    dataType1 tt;
    string s=type(tt,aim);
    string res= "(";
    res = res+s+", ";
    dataType2 ttt;
    s=type(ttt,aim);
    res=res+s+')';
    return res;
  }


}

#endif
