CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C                  projet Estime, INRIA-Rocquencourt                  C
C                                                                     C
C  Copyright 2004-2011 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C $Id: store_value.F,v 1.4 2011-03-09 12:50:16 fclement Exp $

C     
C     Routines to store data in the low-level memory from their number.
C     
      
#include "pio_constants.h"
      
      subroutine seq_store_communication_name(sbr, io, name, len)
C     Stores the communication name, ie the task name or the service
C     name, a string of length nlen, into the memory.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), name*(*)
      integer io, len
      
C     Local variables.
      integer bn, en
      
C     External function.
      integer get_store_ptr
      
      bn = get_store_ptr(io)
      
      call seq_store_string(sbr, io, name, 1, len)
      
      en = get_store_ptr(io)
      
      call store_communication_name_begin(sbr, io, bn)
      call store_communication_name_end(sbr, io, en)
      
      return
      end
      
      
      subroutine seq_store_string_value(sbr, io, sdatum, slen)
C     Stores the string value sdatum of length slen into memory.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), sdatum*MAX_STRING_LENGTH
      integer io, slen
      
      call seq_store_string(sbr, io, sdatum, 1, slen)
      call store_value_len(sbr, io, slen)
      
      return
      end
      
      
      subroutine seq_store_int_value(sbr, io, idatum)
C     Stores the integer value idatum into memory.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, idatum
      
      call seq_store_int(sbr, io, idatum)
      
      return
      end
      
      
      subroutine seq_store_float_value(sbr, io, fdatum)
C     Stores the float value fdatum into memory.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io
      FLOAT fdatum
      
      call seq_store_float(sbr, io, fdatum)
      
      return
      end
      
      
      subroutine seq_store_string_compound_value
     +     (sbr, io, n, vlen, svdatum)
C     Stores the string vector value svdatum of size n, and of which
C     component i has length vlen(i), into memory.
      
      implicit none
      
C     Input arguments.
      character sbr*(*), svdatum(*)*(*)
      integer io, n, vlen(*)
      
C     Local variable.
      integer i
      
      call seq_store_string_vect(sbr, io, svdatum, vlen, 1, n)
      do i = 1, n
         call store_value_len(sbr, io, vlen(i))
      enddo
      
      return
      end
      
      
      subroutine seq_store_int_compound_value(sbr, io, n, ivdatum)
C     Stores the integer vector value ivdatum of size n into memory.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, n, ivdatum(*)
      
      call seq_store_int_vect(sbr, io, ivdatum, 1, n)
      
      return
      end
      
      
      subroutine seq_store_float_compound_value(sbr, io, n, fvdatum)
C     Stores the float vector value fvdatum of size n into memory.
      
      implicit none
      
C     Input arguments.
      character sbr*(*)
      integer io, n
      FLOAT fvdatum(*)
      
      call seq_store_float_vect(sbr, io, fvdatum, 1, n)
      
      return
      end
